﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Infragistics.Documents.Excel;
using System.IO;
using System.Reflection;
using Infragistics.Controls.Interactions;
using System.Collections;
using Infragistics;
using Microsoft.Win32;

namespace Excel_Engine_WPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //the global workbook
        Workbook workbook;
        // the one loaded from an imported file
        Workbook loaded;
        // the selected by the user table to display
        string selected;

        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Generate a workbook with 1 Worksheet and populate and define two tables, name them and apply filtering and sorting options.
        /// </summary>
        private void GenerateWorkbook(object sender, RoutedEventArgs e)
        {
            //create the workbook
            workbook = new Workbook(WorkbookFormat.Excel2007);
            //create the worksheet
            Worksheet worksheet = workbook.Worksheets.Add("NorthwindCustomers");
            //populate headers
            PropertyInfo[] properties = typeof(Customer).GetProperties();
            for (int i = 0; i < 10; i++)
            {
                worksheet.Rows[0].Cells[i].Value = properties[i].Name;
            }

            //populate Customer data 
            NorthwindEntities nwContext = new NorthwindEntities();
            var customers = nwContext.Customers.ToList();
            //rows
            for (int i = 0; i < nwContext.Customers.Count(); i++)
            {
                WorksheetRow row = worksheet.Rows[i + 1]; //+1 to skip the header row
                //columns
                for (int j = 0; j < 10; j++)
                {
                    row.Cells[j].Value = properties[j].GetValue(customers.ElementAt(i), null);
                }
            }
            // define the region
            WorksheetRegion region = new WorksheetRegion(worksheet, 0, 0, nwContext.Customers.Count(), 9);
            // format as table and name it
            WorksheetTable table = region.FormatAsTable(true);
            table.Name = "Customers";
            // apply filter
            table.Columns["Country"].ApplyCustomFilter(new Infragistics.Documents.Excel.Filtering.CustomFilterCondition(Infragistics.Documents.Excel.Filtering.ExcelComparisonOperator.BeginsWith, "U"));


            // Place the SUppliers table *under* the customers one
            //populate headers
            PropertyInfo[] supplierProperties = typeof(Supplier).GetProperties();
            for (int i = 0; i < 10; i++)
            {
                worksheet.Rows[94].Cells[i].Value = supplierProperties[i].Name;
            }

            var suppliers = nwContext.Suppliers.ToList();
            //rows
            for (int i = 95; i < (nwContext.Suppliers.Count() + 95); i++)
            {
                WorksheetRow row = worksheet.Rows[i];
                //columns
                for (int j = 0; j < 10; j++)
                {
                    row.Cells[j].Value = supplierProperties[j].GetValue(suppliers.ElementAt(i - 95), null);
                }
            }
            WorksheetRegion supplierRegion = new WorksheetRegion(worksheet, 94, 0, nwContext.Suppliers.Count() + 94, 9);
            WorksheetTable supplierTable = supplierRegion.FormatAsTable(true);
            supplierTable.Name = "Suppliers";
            //apply sorting condition
            supplierTable.Columns["Phone"].SortCondition = new Infragistics.Documents.Excel.Sorting.OrderedSortCondition(Infragistics.Documents.Excel.Sorting.SortDirection.Ascending);
            workbook.CustomizeLooks();
            btnGenerateFile.IsEnabled = true;

        }

        /// <summary>
        /// Save the workbook to a file
        /// </summary>
        private void GenerateExcelFile(object sender, RoutedEventArgs e)
        {
            // Create Save File Dialog
            SaveFileDialog savefileDialog = new SaveFileDialog { Filter = "Excel files|*.xlsx", DefaultExt = "xlsx" };

            // Display the dialog. If the user selects a file of the correct format 
            // and clicks Save in the dialog, save the file using the Save method of the Workbook object.
            if (savefileDialog.ShowDialog() == true)
            {
                try
                {
                    Stream stream = savefileDialog.OpenFile();
                    workbook.Save(stream);
                    stream.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        /// <summary>
        /// Import data from an Excel file, look for tables and display dialog with their names if more than 1
        /// </summary>
        private void ImportExcelFile(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            if (openFileDialog.ShowDialog() == true)
            {
                try
                {
                    using (Stream stream = openFileDialog.OpenFile())
                    {
                        loaded = Workbook.Load(stream);
                        if (loaded.Worksheets[0].Tables.Count > 1)
                        {
                            //create a XamDialogWindow with the table names and let the user pick which one to see
                            XamDialogWindow window = new XamDialogWindow() { Height = 200, Width = 300, StartupPosition = StartupPosition.Center };
                            window.ModalBackgroundEffect = null;
                            window.IsModal = true;
                            TextBlock message = new TextBlock() { Height = 30, Width = 300 };
                            message.Text += "Found " + loaded.Worksheets[0].Tables.Count + " tables. Choose the one to be displayed:";
                            TextBlock message2 = new TextBlock() { Height = 30, Width = 300 };
                            message2.Text += "The Adress cell has text: " + loaded.Worksheets[0].GetCell("Adress").GetText();
                            ComboBox combo = new ComboBox() { ItemsSource = loaded.Worksheets[0].Tables.Select(x => x.Name), Height = 30, Width = 200 };
                            combo.SelectionChanged += new SelectionChangedEventHandler(combo_SelectionChanged);
                            Button ok = new Button() { Content = "OK", Height = 30, Width = 50 };
                            ok.Click += new RoutedEventHandler(ok_Click);
                            StackPanel panel = new StackPanel();
                            panel.Children.Add(message2);
                            panel.Children.Add(message);
                            panel.Children.Add(combo);
                            panel.Children.Add(ok);
                            window.Content = panel;
                            this.LayoutRoot.Children.Add(window);
                        }
                        else if (loaded.Worksheets[0].Tables.Count == 1)
                        {
                            selected = loaded.Worksheets[0].Tables.FirstOrDefault().Name;
                            FeedDataToGrid();
                        }
                        else
                        {
                            MessageBox.Show("Couldn't find any tables in this file");
                        }

                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Something went terribly wrong here: " + ex.Message);
                }
            }
        }

        void combo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            selected = (sender as ComboBox).SelectedValue.ToString();
        }

        void ok_Click(object sender, RoutedEventArgs e)
        {
            FeedDataToGrid();
            this.LayoutRoot.Children.OfType<XamDialogWindow>().First().Close();
        }

        /// <summary>
        /// Reads selected table data and sets it as data context for the grid
        /// </summary>
        private void FeedDataToGrid()
        {
            var table = loaded.GetTable(selected);
            if (table != null)
            {
                int rows = table.DataAreaRegion.Count() / 10;
                if (selected == "Customers")
                {
                    List<Customer> data = new List<Customer>();
                    PropertyInfo[] properties = typeof(Customer).GetProperties();
                    //rows
                    for (int i = 0; i < rows; i++)
                    {
                        Customer item = new Customer();
                        //columns
                        for (int j = 0; j < 10; j++)
                        {
                            properties[j].SetValue(item, (string)table.DataAreaRegion.ElementAt(i * 10 + j).Value, null);
                        }
                        data.Add(item);
                    }
                    this.xamGrid.ItemsSource = data;
                }
                if (selected == "Suppliers")
                {
                    List<Supplier> data = new List<Supplier>();
                    PropertyInfo[] properties = typeof(Supplier).GetProperties();
                    //rows
                    for (int i = 0; i < rows; i++)
                    {
                        Supplier item = new Supplier();
                        //columns
                        properties[0].SetValue(item, Convert.ToInt32(table.DataAreaRegion.ElementAt(i * 10).Value), null);
                        for (int j = 1; j < 10; j++)
                        {
                            properties[j].SetValue(item, table.DataAreaRegion.ElementAt(i * 10 + j).Value, null);
                        }
                        data.Add(item);
                    }
                    this.xamGrid.ItemsSource = data;
                }
            }
        }
    }

    static class Customization
    {
        /// <summary>
        /// Extension method to hold all the workbook style and format modifications.
        /// </summary>
        public static void CustomizeLooks(this Workbook workbook)
        {
            //set global defaults
            workbook.Styles.NormalStyle.StyleFormat.Font.Name = "Segoe UI";
            workbook.Styles.NormalStyle.StyleFormat.Font.ColorInfo = new WorkbookColorInfo(System.Drawing.Color.FromArgb(255, 40, 170, 170));
            workbook.Styles.NormalStyle.StyleFormat.Alignment = HorizontalCellAlignment.Center;

            // apply nautral style to the 2nd cell on the 5th row
            workbook.Worksheets[0].Rows[4].Cells[1].CellFormat.Style = workbook.Styles["Neutral"];

            //format options
            workbook.Worksheets[0].Rows[4].Cells[1].CellFormat.FormatOptions = WorksheetCellFormatOptions.ApplyFontFormatting | WorksheetCellFormatOptions.ApplyAlignmentFormatting;

            //access the tables by names
            //Note: all styles have a 'TableStyle' prefix , followd by light, medium or dark and index
            WorksheetTable customers = workbook.GetTable("Customers");
            if (workbook.GetTable("Customers") != null)
            {
                workbook.GetTable("Customers").Style = workbook.StandardTableStyles["TableStyleMedium16"];
            }
            if (workbook.GetTable("Suppliers") != null)
            {
                workbook.GetTable("Suppliers").Style = workbook.StandardTableStyles["TableStyleDark3"];
            }

            var test = workbook.StandardTableStyles;
            // add color to the palette
            workbook.Palette[55] = System.Drawing.Color.FromArgb(255, 40, 170, 170);
            //named reference:
            NamedReference reference = workbook.NamedReferences.Add("Adress", "=NorthwindCustomers!A1", workbook.Worksheets.FirstOrDefault());
            //access it either from the worksheet's method or from the 
            //reference.ReferencedCell  
            //property too
            workbook.Worksheets[0].GetCell("Adress").CellFormat.Fill = new CellFillPattern(new WorkbookColorInfo(System.Drawing.Color.DarkGray), new WorkbookColorInfo(System.Drawing.Color.Cyan), FillPatternStyle.HorizontalStripe);
        }

    }
}
